@@ -17,20 +17,15 @@
 package tests
 
 import (
-	"path/filepath"
 	"testing"
 )
 
 func TestRLP(t *testing.T) {
-	err := RunRLPTest(filepath.Join(rlpTestDir, "rlptest.json"), nil)
-	if err != nil {
-		t.Fatal(err)
-	}
-}
-
-func TestRLP_invalid(t *testing.T) {
-	err := RunRLPTest(filepath.Join(rlpTestDir, "invalidRLPTest.json"), nil)
-	if err != nil {
-		t.Fatal(err)
-	}
+	t.Parallel()
+	tm := new(testMatcher)
+	tm.walk(t, rlpTestDir, func(t *testing.T, name string, test *RLPTest) {
+		if err := tm.checkFailure(t, test.Run()); err != nil {
+			t.Error(err)
+		}
+	})
 }
